﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;

namespace Achiral
{
    using Achiral.Extension;

    internal static class PermutationImpl
    {
        public static IEnumerable<int[]> EnumerateImpl(int[] array, Func<int[], bool> constraint, long skipCount)
        {
            var NumerOfElements = array.Length;
            var Isconstrained = constraint != null;

            if (skipCount < 0)
            {
                throw new ArgumentException("skipCount should not be negative value", "skipCount");
            }

            if (!Isconstrained || constraint(array))
            {
                if (skipCount > 0)
                {
                    --skipCount;
                }
                else
                {
                    var dest = new int[NumerOfElements];
                    for (int q = 0; q < array.Length; ++q) { dest[q] = array[q]; }
                    yield return dest;
                }
            }

        LOOP:
            for (int i = array.Length - 2; 0 <= i; --i)
            {
                var next_value = array[i];
                if (next_value < array[i + 1])
                {
                    for (int k = array.Length - 1; 0 <= k; --k)
                    {
                        if (next_value < array[k])
                        {
                            {
                                var tmp = array[k];
                                array[k] = array[i];
                                array[i] = tmp;
                            }

                            // can be replaced to: Array.Reverse(array, i + 1, array.Length - i - 1);
                            for (int m = 1; m < (array.Length - i + 1) / 2; ++m)
                            {
                                var tmp = array[i + m];
                                array[i + m] = array[array.Length - m];
                                array[array.Length - m] = tmp;
                            }
                            if (!Isconstrained || constraint(array))
                            {
                                if (skipCount > 0)
                                {
                                    --skipCount;
                                }
                                else
                                {
                                    var dest = new int[NumerOfElements];
                                    for (int q = 0; q < array.Length; q++) { dest[q] = array[q]; }
                                    yield return dest;
                                }
                            }
                            goto LOOP;
                        }
                    }
                }
            }
        }

        public static IEnumerable<int[]> EnumerateReverseImpl(int[] array, Func<int[], bool> constraint, long skipCount)
        {
            var NumerOfElements = array.Length;
            var Isconstrained = constraint != null;

            if (skipCount < 0)
            {
                throw new ArgumentException("skipCount should not be negative value", "skipCount");
            }

            if (!Isconstrained || constraint(array))
            {
                if (skipCount > 0)
                {
                    --skipCount;
                }
                else
                {
                    var dest = new int[NumerOfElements];
                    for (int q = 0; q < array.Length; ++q) { dest[q] = array[q]; }
                    yield return dest;
                }
            }

        LOOP:
            for (int i = array.Length - 2; 0 <= i; --i)
            {
                var next_value = array[i];
                if (next_value > array[i + 1])
                {
                    for (int k = array.Length - 1; 0 <= k; --k)
                    {
                        if (next_value > array[k])
                        {
                            {
                                var tmp = array[k];
                                array[k] = array[i];
                                array[i] = tmp;
                            }

                            // can be replaced to: Array.Reverse(array, i + 1, array.Length - i - 1);
                            for (int m = 1; m < (array.Length - i + 1) / 2; ++m)
                            {
                                var tmp = array[i + m];
                                array[i + m] = array[array.Length - m];
                                array[array.Length - m] = tmp;
                            }
                            if (!Isconstrained || constraint(array))
                            {
                                if (skipCount > 0)
                                {
                                    --skipCount;
                                }
                                else
                                {
                                    var dest = new int[NumerOfElements];
                                    for (int q = 0; q < array.Length; q++) { dest[q] = array[q]; }
                                    yield return dest;
                                }
                            }
                            goto LOOP;
                        }
                    }
                }
            }
        }
    }

    public class PermutationSource : IEnumerable<int[]>
    {
        private static readonly long[] FactorialsCache = new []
        {
            1L, 1L, 2L, 6L, 24L, 120L, 720L, 5040L, 40320L,
            362880L, 3628800L, 39916800L, 479001600L, 6227020800L,
            87178291200L, 1307674368000L, 20922789888000L, 355687428096000L,
            6402373705728000L, 121645100408832000L, 2432902008176640000L
        };
        public readonly int NumerOfElements;
        public readonly bool Forward;
        public readonly Expression<Func<int[], bool>> Constraint;
        public bool Isconstrained { get { return Constraint != null; } }
        private Func<int[], bool> _constraintImpl = null;
        private Func<int[], bool> ConstraintImpl
        {
            get
            {
                if (_constraintImpl == null && Constraint != null)
                {
                    _constraintImpl = Constraint.Compile();
                }
                return _constraintImpl;
            }
        }

        internal PermutationSource(int numberOfElements)
            : this(numberOfElements, true, null)
        {
        }
        internal PermutationSource(int numberOfElements, bool forward, Expression<Func<int[], bool>> constraint)
        {
            if (numberOfElements < 1)
            {
                throw new ArgumentException("numberOfElements should be positive integer", "numberOfElements");
            }

            NumerOfElements = numberOfElements;
            Forward = forward;
            Constraint = constraint;
        }
        public IEnumerable<int[]> AsEnumerable()
        {
            return this;
        }
        public PermutationSource Reverse()
        {
            return new PermutationSource(NumerOfElements, !Forward, Constraint);
        }
        public PermutationSource Where(Expression<Func<int[], bool>> predicateExpression)
        {
            return new PermutationSource(NumerOfElements, Forward, Constraint.AndAlso(predicateExpression));
        }
        public int[] Min()
        {
            return (Forward ? this : this.Reverse()).AsEnumerable().First();
        }
        public int[] Max()
        {
            return (Forward ? this.Reverse() : this).AsEnumerable().First();
        }
        public int[] First(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).First();
        }
        public int[] FirstOrDefault(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).FirstOrDefault();
        }
        public int[] SingleOrDefault(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).SingleOrDefault();
        }
        public int[] Last()
        {
            return this.Reverse().First();
        }
        public int[] Last(Expression<Func<int[], bool>> predicate)
        {
            return this.Reverse().First(predicate);
        }
        public int[] LastOrDefault(Expression<Func<int[], bool>> predicate)
        {
            return this.Reverse().FirstOrDefault(predicate);
        }
        public int[] ElementAt(int index)
        {
            return this.ElementAt((long)index);
        }
        public int[] ElementAt(long index)
        {
            var ret = this.ElementAtOrDefault(index);
            if(ret == null)
            {
                throw new ArgumentException("index out of range", "index");
            }
            return ret;
        }
        public int[] ElementAtOrDefault(int index)
        {
            return this.ElementAtOrDefault((long) index);
        }
        public int[] ElementAtOrDefault(long index)
        {
            if (Isconstrained || (FactorialsCache.Length <= NumerOfElements))
            {
                return this.Skip(index).FirstOrDefault();
            }

            if(index < 0 || FactorialsCache[NumerOfElements] <= index)
            {
                return null;
            }

            index = Forward ? index : FactorialsCache[NumerOfElements] - 1 - index;

            // http://msdn.microsoft.com/en-us/library/aa302371.aspx#permutat_topic3
            var factoradic = new int[NumerOfElements];
            var rest = index;
            for (int i = 0; i < factoradic.Length; ++i)
            {
                var basis = FactorialsCache[NumerOfElements - 1 - i];
                factoradic[i] = checked((int)(rest / basis));
                rest %= basis;
            }
            var temp = new int[NumerOfElements];
            for (int i = 0; i < temp.Length; ++i)
            {
                temp[i] = ++factoradic[i];
            }
            var array = new int[NumerOfElements];
            array[array.Length - 1] = 1;
            for (int i = array.Length - 2; i >= 0; --i)
            {
                array[i] = temp[i];
                for (int j = i + 1; j < array.Length; ++j)
                {
                    if (array[j] >= array[i])
                    {
                        ++array[j];
                    }
                }
            }
            for (int i = 0; i < array.Length; ++i)
            {
                --array[i];
            }
            return array;
        }
        public IEnumerable<int[]> Skip(int count)
        {
            return this.Skip((long) count);
        }
        public IEnumerable<int[]> Skip(long count)
        {
            var array = new int[NumerOfElements];

            if (Isconstrained)
            {
                if (Forward)
                {
                    for (int q = 0; q < array.Length; ++q) { array[q] = q; }
                    return PermutationImpl.EnumerateImpl(array, this.ConstraintImpl, count);
                }
                else
                {
                    for (int q = 0; q < array.Length; ++q) { array[q] = NumerOfElements - 1 - q; }
                    return PermutationImpl.EnumerateReverseImpl(array, this.ConstraintImpl, count);
                }
            }
            var src = ElementAt(count);
            for (int q = 0; q < array.Length; ++q) { array[q] = src[q]; }
            return Forward ? PermutationImpl.EnumerateImpl(src, this.ConstraintImpl, 0) : PermutationImpl.EnumerateReverseImpl(src, this.ConstraintImpl, 0);
        }
        public bool All(Expression<Func<int[], bool>> predicate)
        {
            return !this.Where(predicate.Not()).AsEnumerable().Any();
        }
        public bool Any()
        {
            return Isconstrained ? this.AsEnumerable().Any() : true;
        }
        public bool Any(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).AsEnumerable().Any();
        }
        public int Count()
        {
            if (Isconstrained)
            {
                return this.AsEnumerable().Count();
            }
            if (NumerOfElements >= FactorialsCache.Length)
            {
                throw new NotSupportedException("Too large element count");
            }
            return checked((int)FactorialsCache[NumerOfElements]);
        }
        public int Count(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).Count();
        }
        public long LongCount()
        {
            if (Isconstrained)
            {
                return this.AsEnumerable().LongCount();
            }
            if (FactorialsCache.Length <= NumerOfElements)
            {
                throw new NotSupportedException("Too large element count");
            }
            return FactorialsCache[NumerOfElements];
        }
        public long LongCount(Expression<Func<int[], bool>> predicate)
        {
            return this.Where(predicate).Count();
        }
        public IEnumerator<int[]> GetEnumerator()
        {
            var array = new int[NumerOfElements];
            if(Forward)
            {
                for (int q = 0; q < array.Length; ++q) { array[q] = q; }
                return PermutationImpl.EnumerateImpl(array, this.ConstraintImpl, 0).GetEnumerator();
            }
            else
            {
                for (int q = 0; q < array.Length; ++q) { array[q] = NumerOfElements - 1 - q; }
                return PermutationImpl.EnumerateReverseImpl(array, this.ConstraintImpl, 0).GetEnumerator();
            }
        }
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }
}

