﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Collections.Generic;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static IEnumerable<T> Unfold<T>(this Func<T, T> func, T initial)
        {
            AssertNonNullArgument(func, "func");
            var value = initial;
            yield return value;
            while (true)
            {
                value = func(value);
                yield return value;
            }
        }

        public static IEnumerable<T> CascadeDepthFirst<T>(this Func<T, IEnumerable<T>> func, T initial)
        {
            AssertNonNullArgument(func, "func");
            return Make.Sequence(initial).CascadeDepthFirst(func);
        }
        public static IEnumerable<TResult> CascadeDepthFirst<T, TResult>(this Func<T, IEnumerable<T>> func, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(func, "func");
            return Make.Sequence(initial).CascadeDepthFirst(func, selector);
        }
        public static IEnumerable<T> CascadeBreadthFirst<T>(this Func<T, IEnumerable<T>> func, T initial)
        {
            AssertNonNullArgument(func, "func");
            return Make.Sequence(initial).CascadeBreadthFirst(func);
        }
        public static IEnumerable<TResult> CascadeBreadthFirst<T, TResult>(this Func<T, IEnumerable<T>> func, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(func, "func");
            return Make.Sequence(initial).CascadeBreadthFirst(func, selector);
        }
    }
}