﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using System.Linq.Expressions;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {

        public static TDelegate CreateDelegate<TDelegate>(this MethodInfo method)
            where TDelegate:class
        {
            return method.CreateDelegateFromType(typeof(TDelegate)) as TDelegate;
        }
        public static Delegate CreateDelegateFromType(this MethodInfo method, Type delegateType)
        {
            AssertNonNullArgument(method, "method");
            return Delegate.CreateDelegate(delegateType, null, method);
        }

        public static TDelegate CreateDelegate<TDelegate, TReceiver>(this MethodInfo method, TReceiver reciever)
            where TDelegate : class
        {
            return method.CreateDelegateFromType(typeof(TDelegate), reciever) as TDelegate;
        }
        public static Delegate CreateDelegateFromType<TReceiver>(this MethodInfo method, Type delegateType, TReceiver reciever)
        {
            AssertNonNullArgument(method, "method");
            return Delegate.CreateDelegate(delegateType, reciever, method);
        }


        public static Action CreateAction(this MethodInfo method)
        {
            return method.CreateDelegate<Action>();
        }
        public static Action<T1> CreateAction<T1>(this MethodInfo method)
        {
            return method.CreateDelegate<Action<T1>>();
        }
        public static Action<T1, T2> CreateAction<T1, T2>(this MethodInfo method)
        {
            return method.CreateDelegate<Action<T1, T2>>();
        }
        public static Action<T1, T2, T3> CreateAction<T1, T2, T3>(this MethodInfo method)
        {
            return method.CreateDelegate<Action<T1, T2, T3>>();
        }
        public static Action<T1, T2, T3, T4> CreateAction<T1, T2, T3, T4>(this MethodInfo method)
        {
            return method.CreateDelegate<Action<T1, T2, T3, T4>>();
        }

        public static Func<TResult> CreateFunc<TResult>(this MethodInfo method)
        {
            return method.CreateDelegate<Func<TResult>>();
        }
        public static Func<T1, TResult> CreateFunc<T1, TResult>(this MethodInfo method)
        {
            return method.CreateDelegate<Func<T1, TResult>>();
        }
        public static Func<T1, T2, TResult> CreateFunc<T1, T2, TResult>(this MethodInfo method)
        {
            return method.CreateDelegate<Func<T1, T2, TResult>>();
        }
        public static Func<T1, T2, T3, TResult> CreateFunc<T1, T2, T3, TResult>(this MethodInfo method)
        {
            return method.CreateDelegate<Func<T1, T2, T3, TResult>>();
        }
        public static Func<T1, T2, T3, T4, TResult> CreateFunc<T1, T2, T3, T4, TResult>(this MethodInfo method)
        {
            return method.CreateDelegate<Func<T1, T2, T3, T4, TResult>>();
        }

        public static Delegate CreateAction(this MethodInfo method, params Type[] argumentTypes)
        {
            AssertNonNullArgument(argumentTypes, "argumentTypes");

            return method.CreateDelegateFromType(Expression.GetActionType(argumentTypes));
        }
        public static Delegate CreateFunc(this MethodInfo method, params Type[] argumentsAndResultTypes)
        {
            AssertNonNullArgument(argumentsAndResultTypes, "argumentsAndResultTypes");

            return method.CreateDelegateFromType(Expression.GetFuncType(argumentsAndResultTypes));
        }
    }
}