﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static bool IsNullOrEmpty(this string str)
        {
            return string.IsNullOrEmpty(str);
        }
        public static string Join(this string[] source, string separator)
        {
            AssertNonNullArgument(source, "source");

            return string.Join(separator, source);
        }
        public static Regex ToRegex(this string pattern)
        {
            AssertNonNullArgument(pattern, "pattern");
            return new Regex(pattern);
        }
        public static Regex ToRegex(this string pattern, RegexOptions options)
        {
            AssertNonNullArgument(pattern, "pattern");
            return new Regex(pattern, options);
        }
        public static Match RegexMatch(this string pattern, string input)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            return new Regex(pattern).Match(input);
        }
        public static Match RegexMatch(this string pattern, string input, RegexOptions options)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            return new Regex(pattern, options).Match(input);
        }
        public static Maybe<TResult> RegexMatch<TResult>(this string pattern, string input, Func<Match, TResult> selector)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            AssertNonNullArgument(selector, "selector");
            return new Regex(pattern).Match(input).Select(selector);
        }
        public static Maybe<TResult> RegexMatch<TResult>(this string pattern, string input, RegexOptions options, Func<Match, TResult> selector)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            AssertNonNullArgument(selector, "selector");
            return new Regex(pattern, options).Match(input).Select(selector);
        }
        public static MatchCollection RegexMatches(this string pattern, string input)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            return new Regex(pattern).Matches(input);
        }
        public static MatchCollection RegexMatches(this string pattern, string input, RegexOptions options)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            return new Regex(pattern, options).Matches(input);
        }
        public static IEnumerable<TResult> RegexMatches<TResult>(this string pattern, string input, Func<Match, TResult> selector)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            AssertNonNullArgument(selector, "selector");
            return new Regex(pattern).Matches(input).Select(selector);
        }
        public static IEnumerable<TResult> RegexMatches<TResult>(this string pattern, string input, RegexOptions options, Func<Match, TResult> selector)
        {
            AssertNonNullArgument(pattern, "pattern");
            AssertNonNullArgument(input, "input");
            AssertNonNullArgument(selector, "selector");
            return new Regex(pattern, options).Matches(input).Select(selector);
        }
    }
}