﻿using System;
using System.Linq;
using System.Reflection;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static ConstructorInfo[] GetAllDeclaredConstructors(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetConstructors(BindingFlagsForAllDeclaredElement);
        }
        public static ConstructorInfo[] GetAllFlattenConstructors(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetConstructors(BindingFlagsForAllFlattenElement);
        }
        public static FieldInfo[] GetAllDeclaredFields(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetFields(BindingFlagsForAllDeclaredElement);
        }
        public static FieldInfo[] GetAllFlattenFields(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetFields(BindingFlagsForAllFlattenElement);
        }
        public static MemberInfo[] GetAllDeclaredMembers(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMembers(BindingFlagsForAllDeclaredElement);
        }
        public static MemberInfo[] GetAllFlattenMembers(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMembers(BindingFlagsForAllFlattenElement);
        }
        public static MethodInfo[] GetAllDeclaredMethods(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMethods(BindingFlagsForAllDeclaredElement);
        }
        public static MethodInfo[] GetAllFlattenMethods(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMethods(BindingFlagsForAllFlattenElement);
        }
        public static PropertyInfo[] GetAllDeclaredProperties(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetProperties(BindingFlagsForAllDeclaredElement);
        }
        public static PropertyInfo[] GetAllFlattenProperties(this Type type)
        {
            AssertNonNullArgument(type, "type");
            return type.GetProperties(BindingFlagsForAllFlattenElement);
        }
        private static readonly BindingFlags
            BindingFlagsForAllDeclaredElement =
                        BindingFlags.Static
                      | BindingFlags.Instance
                      | BindingFlags.NonPublic
                      | BindingFlags.Public
                      | BindingFlags.DeclaredOnly;
        private static readonly BindingFlags
            BindingFlagsForAllFlattenElement =
                        BindingFlags.Static
                      | BindingFlags.Instance
                      | BindingFlags.NonPublic
                      | BindingFlags.Public
                      | BindingFlags.FlattenHierarchy;

        public static int GetHierarchyLevel(this Type type)
        {
            AssertNonNullArgument(type, "type");
            for (int level = 0; ; ++level )
            {
                if (type.BaseType == null || type.BaseType == type)
                    return level;
                else
                    type = type.BaseType;
            }
        }
    }
}