﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Collections.Generic;

namespace Achiral
{
    using Achiral.Extension;
    using System.Runtime.InteropServices;

    // Declaration
    [StructLayout(LayoutKind.Auto)]
    public partial struct Tuple<T1, T2>
    {
        public readonly T1 Item1;
        public readonly T2 Item2;
        public Tuple(T1 item1, T2 item2)
        {
            this.Item1 = item1;
            this.Item2 = item2;
        }
        public static Tuple<T1, T2> Default
        {
            get
            {
                return new Tuple<T1, T2>(default(T1), default(T2));
            }
        }
    }

    [StructLayout(LayoutKind.Auto)]
    public partial struct Tuple<T1, T2, T3>
    {
        public readonly T1 Item1;
        public readonly T2 Item2;
        public readonly T3 Item3;
        public Tuple(T1 item1, T2 item2, T3 item3)
        {
            this.Item1 = item1;
            this.Item2 = item2;
            this.Item3 = item3;
        }
        public static Tuple<T1, T2, T3> Default
        {
            get
            {
                return new Tuple<T1, T2, T3>(default(T1), default(T2), default(T3));
            }
        }
    }

    [StructLayout(LayoutKind.Auto)]
    public partial struct Tuple<T1, T2, T3, T4>
    {
        public readonly T1 Item1;
        public readonly T2 Item2;
        public readonly T3 Item3;
        public readonly T4 Item4;
        public Tuple(T1 item1, T2 item2, T3 item3, T4 item4)
        {
            this.Item1 = item1;
            this.Item2 = item2;
            this.Item3 = item3;
            this.Item4 = item4;
        }
        public static Tuple<T1, T2, T3, T4> Default
        {
            get
            {
                return new Tuple<T1, T2, T3, T4>(default(T1), default(T2), default(T3), default(T4));
            }
        }
    }

    // Select
    public partial struct Tuple<T1, T2>
    {
        public TResult Select<TResult>(Func<T1, T2, TResult> selector)
        {
            return selector(this.Item1, this.Item2);
        }
    }
    public partial struct Tuple<T1, T2, T3>
    {
        public TResult Select<TResult>(Func<T1, T2, T3, TResult> selector)
        {
            return selector(this.Item1, this.Item2, this.Item3);
        }
    }
    public partial struct Tuple<T1, T2, T3, T4>
    {
        public TResult Select<TResult>(Func<T1, T2, T3, T4, TResult> selector)
        {
            return selector(this.Item1, this.Item2, this.Item3, this.Item4);
        }
    }

    // DebuggerDisplay
    [DebuggerDisplay("({Item1}, {Item2})")]
    public partial struct Tuple<T1, T2> { }
    [DebuggerDisplay("({Item1}, {Item2}, {Item3})")]
    public partial struct Tuple<T1, T2, T3> { }
    [DebuggerDisplay("({Item1}, {Item2}, {Item3}, {Item4})")]
    public partial struct Tuple<T1, T2, T3, T4> { }

    // Make
    public static partial class Make
    {
        public static Tuple<T1, T2> Tuple<T1, T2>(T1 item1, T2 item2)
        {
            return new Tuple<T1, T2>(item1, item2);
        }
        public static Tuple<T1, T2, T3> Tuple<T1, T2, T3>(T1 item1, T2 item2, T3 item3)
        {
            return new Tuple<T1, T2, T3>(item1, item2, item3);
        }
        public static Tuple<T1, T2, T3, T4> Tuple<T1, T2, T3, T4>(T1 item1, T2 item2, T3 item3, T4 item4)
        {
            return new Tuple<T1, T2, T3, T4>(item1, item2, item3, item4);
        }
    }

    // IEquatable Implimentation (Generated by ReSharper 3.1)

    public partial struct Tuple<T1, T2> : IEquatable<Tuple<T1, T2>>
    {
        public static bool operator !=(Tuple<T1, T2> tuple1, Tuple<T1, T2> tuple2)
        {
            return !tuple1.Equals(tuple2);
        }

        public static bool operator ==(Tuple<T1, T2> tuple1, Tuple<T1, T2> tuple2)
        {
            return tuple1.Equals(tuple2);
        }

        public bool Equals(Tuple<T1, T2> tuple)
        {
            return Equals(Item1, tuple.Item1) && Equals(Item2, tuple.Item2);
        }

        public override bool Equals(object obj)
        {
            if (!(obj is Tuple<T1, T2>)) return false;
            return Equals((Tuple<T1, T2>)obj);
        }

        public override int GetHashCode()
        {
            return (Item1 != null ? Item1.GetHashCode() : 0) + 29 * (Item2 != null ? Item2.GetHashCode() : 0);
        }
    }


    public partial struct Tuple<T1, T2, T3> : IEquatable<Tuple<T1, T2, T3>>
    {
        public static bool operator !=(Tuple<T1, T2, T3> tuple1, Tuple<T1, T2, T3> tuple2)
        {
            return !tuple1.Equals(tuple2);
        }

        public static bool operator ==(Tuple<T1, T2, T3> tuple1, Tuple<T1, T2, T3> tuple2)
        {
            return tuple1.Equals(tuple2);
        }

        public bool Equals(Tuple<T1, T2, T3> tuple)
        {
            if (!Make.Equality(Item1, tuple.Item1)) return false;
            if (!Make.Equality(Item2, tuple.Item2)) return false;
            if (!Make.Equality(Item3, tuple.Item3)) return false;
            return true;
        }

        public override bool Equals(object obj)
        {
            if (!(obj is Tuple<T1, T2, T3>)) return false;
            return Equals((Tuple<T1, T2, T3>)obj);
        }

        public override int GetHashCode()
        {
            int result = Item1 != null ? Item1.GetHashCode() : 0;
            result = 29 * result + (Item2 != null ? Item2.GetHashCode() : 0);
            result = 29 * result + (Item3 != null ? Item3.GetHashCode() : 0);
            return result;
        }
    }

    public partial struct Tuple<T1, T2, T3, T4> : IEquatable<Tuple<T1, T2, T3, T4>>
    {
        public static bool operator !=(Tuple<T1, T2, T3, T4> tuple1, Tuple<T1, T2, T3, T4> tuple2)
        {
            return !tuple1.Equals(tuple2);
        }

        public static bool operator ==(Tuple<T1, T2, T3, T4> tuple1, Tuple<T1, T2, T3, T4> tuple2)
        {
            return tuple1.Equals(tuple2);
        }

        public bool Equals(Tuple<T1, T2, T3, T4> tuple)
        {
            if (!Make.Equality(Item1, tuple.Item1)) return false;
            if (!Make.Equality(Item2, tuple.Item2)) return false;
            if (!Make.Equality(Item3, tuple.Item3)) return false;
            if (!Make.Equality(Item4, tuple.Item4)) return false;
            return true;
        }

        public override bool Equals(object obj)
        {
            if (!(obj is Tuple<T1, T2, T3, T4>)) return false;
            return Equals((Tuple<T1, T2, T3, T4>)obj);
        }

        public override int GetHashCode()
        {
            int result = Item1 != null ? Item1.GetHashCode() : 0;
            result = 29 * result + (Item2 != null ? Item2.GetHashCode() : 0);
            result = 29 * result + (Item3 != null ? Item3.GetHashCode() : 0);
            result = 29 * result + (Item4 != null ? Item4.GetHashCode() : 0);
            return result;
        }
    }

    namespace Extension
    {
        public static partial class MakeWorld
        {
            public static IEnumerable<TResult> SelectAsTuple<T1, T2, TResult>(this IEnumerable<Tuple<T1, T2>> source, Func<T1, T2, TResult> selector)
            {
                return source.Select(item => selector(item.Item1, item.Item2));
            }
            public static IEnumerable<TResult> SelectAsTuple<T1, T2, T3, TResult>(this IEnumerable<Tuple<T1, T2, T3>> source, Func<T1, T2, T3, TResult> selector)
            {
                return source.Select(item => selector(item.Item1, item.Item2, item.Item3));
            }
            public static IEnumerable<TResult> SelectAsTuple<T1, T2, T3, T4, TResult>(this IEnumerable<Tuple<T1, T2, T3, T4>> source, Func<T1, T2, T3, T4, TResult> selector)
            {
                return source.Select(item => selector(item.Item1, item.Item2, item.Item3, item.Item4));
            }

            public static void ForEachAsTuple<T1, T2>(this IEnumerable<Tuple<T1, T2>> source, Action<T1, T2> action)
            {
                source.ForEach(item => action(item.Item1, item.Item2));
            }
            public static void ForEachAsTuple<T1, T2, T3>(this IEnumerable<Tuple<T1, T2, T3>> source, Action<T1, T2, T3> action)
            {
                source.ForEach(item => action(item.Item1, item.Item2, item.Item3));
            }
            public static void ForEachAsTuple<T1, T2, T3, T4>(this IEnumerable<Tuple<T1, T2, T3, T4>> source, Action<T1, T2, T3, T4> action)
            {
                source.ForEach(item => action(item.Item1, item.Item2, item.Item3, item.Item4));
            }

            
        }
    }
}