﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Achiral;
using Achiral.Extension;
using System.IO;

namespace Achiral.Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class PermutationTest
    {
        public PermutationTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void Permutation_OutOfRangeTest()
        {
            try
            {
                var seq = Make.Permutation(0).ToArray();
                Assert.Fail();
            }
            catch (ArgumentException)
            {

            }
            catch (Exception)
            {
                Assert.Fail();
            }
        }

        [TestMethod]
        public void Permutation_SimpleTest()
        {
            var arrays = Make.Permutation(4)
                             .Select(array => array.ToArray())
                             .ToArray();

            var answers = new[]{
                              new[] {0, 1, 2, 3}, new[] {0, 1, 3, 2}, new[] {0, 2, 1, 3}, new[] {0, 2, 3, 1},
                              new[] {0, 3, 1, 2}, new[] {0, 3, 2, 1}, new[] {1, 0, 2, 3}, new[] {1, 0, 3, 2},
                              new[] {1, 2, 0, 3}, new[] {1, 2, 3, 0}, new[] {1, 3, 0, 2}, new[] {1, 3, 2, 0},
                              new[] {2, 0, 1, 3}, new[] {2, 0, 3, 1}, new[] {2, 1, 0, 3}, new[] {2, 1, 3, 0},
                              new[] {2, 3, 0, 1}, new[] {2, 3, 1, 0}, new[] {3, 0, 1, 2}, new[] {3, 0, 2, 1},
                              new[] {3, 1, 0, 2}, new[] {3, 1, 2, 0}, new[] {3, 2, 0, 1}, new[] {3, 2, 1, 0},
                          };

            arrays.ZipWith(answers, (arr1, arr2) => arr1.SequenceEqual(arr2))
                  .ForEach(Assert.IsTrue);
        }

        [TestMethod]
        public void Permutation_SkipTest()
        {
            var arrays = Make.Permutation(4)
                             .Skip(12)
                             .Select(array => array.ToArray())
                             .ToArray();

            var answers = new[]{
                              new[] {2, 0, 1, 3}, new[] {2, 0, 3, 1}, new[] {2, 1, 0, 3}, new[] {2, 1, 3, 0},
                              new[] {2, 3, 0, 1}, new[] {2, 3, 1, 0}, new[] {3, 0, 1, 2}, new[] {3, 0, 2, 1},
                              new[] {3, 1, 0, 2}, new[] {3, 1, 2, 0}, new[] {3, 2, 0, 1}, new[] {3, 2, 1, 0},
                          };

            arrays.ZipWith(answers, (arr1, arr2) => arr1.SequenceEqual(arr2))
                  .ForEach(Assert.IsTrue);
        }

        [TestMethod]
        public void Permutation_ReverseTest()
        {
            var arrays = Make.Permutation(3)
                             .Reverse()
                             .Select(array => array.ToArray())
                             .ToArray();

            var answers = new[]{
                              new[] {2, 1, 0}, new[] {2, 0, 1}, new[] {1, 2, 0},
                              new[] {1, 0, 2}, new[] {0, 2, 1}, new[] {0, 1, 2},
                          };

            arrays.ZipWith(answers, (arr1, arr2) => arr1.SequenceEqual(arr2))
                  .ForEach(Assert.IsTrue);
        }

        [TestMethod]
        public void Permutation_ReverseFirstLastTest()
        {
            var revfirst = Make.Permutation(3)
                               .Reverse()
                               .First();
            var revlast = Make.Permutation(3)
                              .Reverse()
                              .Last();

            Assert.IsTrue(revfirst.SequenceEqual(new[] { 2, 1, 0 }));
            Assert.IsTrue(revlast.SequenceEqual(new[] { 0, 1, 2 }));
        }
        
        [TestMethod]
        public void Permutation_ConditionalFirstLastTest()
        {
            var first = Make.Permutation(3)
                            .First(array => array[0] == 1);

            var last = Make.Permutation(3)
                           .Last(array => array[0] == 1);

            Assert.IsTrue(first.SequenceEqual(new[] { 1, 0, 2 }));
            Assert.IsTrue(last.SequenceEqual(new[] { 1, 2, 0 }));
        }

        [TestMethod]
        public void Permutation_WhereTest()
        {
            var arrays = Make.Permutation(3)
                             .Where(array => array[0] == 0)
                             .Where(array => array[1] == 1)
                             .ToArray();

            var answers = new[]{
                              new[] {0, 1, 2}
                          };

            arrays.ZipWith(answers, (arr1, arr2) => arr1.SequenceEqual(arr2))
                  .ForEach(Assert.IsTrue);
        }

        [TestMethod]
        public void Permutation_ElementAtOrDefaultTest()
        {
            Assert.IsTrue(Make.Permutation(4).ElementAt(4).SequenceEqual(new[] { 0, 3, 1, 2 }));
            Assert.IsTrue(Make.Permutation(4).Reverse().ElementAt(4).SequenceEqual(new[] { 3, 0, 2, 1 }));
            Assert.IsTrue(Make.Permutation(4).Where(array => array[0] == 1).ElementAt(2).SequenceEqual(new[] { 1, 2, 0, 3 }));
            Assert.IsTrue(Make.Permutation(4).Where(array => array[0] == 1).Reverse().ElementAt(2).SequenceEqual(new[] { 1, 2, 3, 0 }));
        }

        [TestMethod]
        public void Permutation_CountTest()
        {
            Assert.AreEqual(Make.Permutation(4).Count(), 24);
            Assert.AreEqual(Make.Permutation(4).Reverse().Count(), 24);
            Assert.AreEqual(Make.Permutation(4).Count(array => array[0] == 0), 6);
            Assert.AreEqual(Make.Permutation(4).Reverse().Count(array => array[0] == 0), 6);

            Assert.AreEqual(Make.Permutation(4).LongCount(), 24L);
            Assert.AreEqual(Make.Permutation(4).Reverse().LongCount(), 24L);
            Assert.AreEqual(Make.Permutation(4).LongCount(array => array[0] == 0), 6L);
            Assert.AreEqual(Make.Permutation(4).Reverse().LongCount(array => array[0] == 0), 6L);
        }

        [TestMethod]
        public void Permutation_MinMaxTest()
        {
            Assert.IsTrue(Make.Permutation(4).Min().SequenceEqual(new[] { 0, 1, 2, 3 }));
            Assert.IsTrue(Make.Permutation(4).Max().SequenceEqual(new[] { 3, 2, 1, 0 }));
            Assert.IsTrue(Make.Permutation(4).Reverse().Min().SequenceEqual(new[] { 0, 1, 2, 3 }));
            Assert.IsTrue(Make.Permutation(4).Reverse().Max().SequenceEqual(new[] { 3, 2, 1, 0 }));
            Assert.IsTrue(Make.Permutation(4).Where(array => array[0] == 1).Min().SequenceEqual(new[] { 1, 0, 2, 3 }));
            Assert.IsTrue(Make.Permutation(4).Where(array => array[0] == 1).Max().SequenceEqual(new[] { 1, 3, 2, 0 }));
            Assert.IsTrue(Make.Permutation(4).Reverse().Where(array => array[0] == 1).Min().SequenceEqual(new[] { 1, 0, 2, 3 }));
            Assert.IsTrue(Make.Permutation(4).Reverse().Where(array => array[0] == 1).Max().SequenceEqual(new[] { 1, 3, 2, 0 }));
        }

        [TestMethod]
        public void Permutation_AnyAllTest()
        {
            Assert.IsTrue(Make.Permutation(100).Any());
            Assert.IsTrue(Make.Permutation(100).Where(_ => true).Any());
            Assert.IsTrue(Make.Permutation(100).Any(array => array[0] >= 0));
            Assert.IsFalse(Make.Permutation(100).All(array => array[0] < 0));
        }
    }
}
