﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Achiral;
using Achiral.Extension;
using System.Text.RegularExpressions;

namespace Achiral.Test
{
    /// <summary>
    /// Summary description for StringTest
    /// </summary>
    [TestClass]
    public class RegexTest
    {
        public RegexTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void Regex_SimpleTest1()
        {
            var pattern = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/";
            var sampleText = @"http://example.com:80/example";
            var resultPattern = "${proto}${port}";
            var ret1 = pattern.RegexMatch(sampleText)
                              .Result(resultPattern);

            Assert.AreEqual(ret1, "http:80");

            var ret2 = pattern.RegexMatch("\n" + sampleText, RegexOptions.Multiline)
                              .Result(resultPattern);

            Assert.AreEqual(ret2, "http:80");
        }

        [TestMethod]
        public void Regex_SimpleTest2()
        {
            var pattern = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/";
            Assert.AreEqual(new Regex(pattern).Options, pattern.ToRegex().Options);
            var opt = RegexOptions.Compiled;
            Assert.AreEqual(new Regex(pattern, opt).Options, pattern.ToRegex(opt).Options);
        }

        [TestMethod]
        public void Regex_SimpleTest3()
        {
            var ret = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/"
                        .RegexMatch(@"http://example.com:80/example")
                        .Result("${proto}${port}");

            Assert.AreEqual(ret, "http:80");
        }

        [TestMethod]
        public void Regex_SimpleTest4()
        {
            var pattern = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/";
            var sampleText = @"http://example.com:80/example";
            var failText = @"aaaa";
            var resultPattern = "${proto}${port}";

            var ret = pattern.RegexMatch(sampleText, match => match.Result(resultPattern));

            Assert.IsFalse(ret.IsNothing);

            ret = pattern.RegexMatch("\n" + sampleText, RegexOptions.Multiline, match => match.Result(resultPattern));

            Assert.IsFalse(ret.IsNothing);

            ret = pattern.RegexMatch(failText, match => match.Result(resultPattern));

            Assert.IsTrue(ret.IsNothing);

            ret = pattern.RegexMatch(failText, RegexOptions.Multiline, match => match.Result(resultPattern));

            Assert.IsTrue(ret.IsNothing);

        }

        [TestMethod]
        public void Regex_MatchesTest1()
        {
            var pattern = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/";
            var sampleText = @"http://example.com:80/example";
            var resultPattern = "${proto}${port}";
            var answer = @"http:80";

            var matches = pattern.RegexMatches(sampleText);

            Assert.AreEqual(matches.Count, 1);
            Assert.AreEqual(matches[0].Result(resultPattern), answer);

            matches = pattern.RegexMatches("\n" + sampleText, RegexOptions.Multiline);

            Assert.AreEqual(matches.Count, 1);
            Assert.AreEqual(matches[0].Result(resultPattern), answer);
        }
        [TestMethod]
        public void Regex_MatchesTest2()
        {
            var pattern = @"^(?<proto>\w+)://[^/]+?(?<port>:\d+)?/";
            var sampleText = @"http://example.com:80/example";
            var failText = @"aaaa";
            var resultPattern = "${proto}${port}";
            var answer = @"http:80";

            var seq = pattern.RegexMatches(sampleText, matches => matches.Result(resultPattern));

            Assert.AreEqual(seq.Count(), 1);
            Assert.AreEqual(seq.First(), answer);

            seq = pattern.RegexMatches("\n" + sampleText, RegexOptions.Multiline, matches => matches.Result(resultPattern));

            Assert.AreEqual(seq.Count(), 1);
            Assert.AreEqual(seq.First(), answer);

            seq = pattern.RegexMatches(failText, matches => matches.Result(resultPattern));

            Assert.IsFalse(seq.Any());

            seq = pattern.RegexMatches(failText, RegexOptions.Multiline, matches => matches.Result(resultPattern));

            Assert.IsFalse(seq.Any());

        }
    }
}
